import colors from 'colors';
import { join, resolve } from 'path';
import { exists, rename, mkdir } from '../utils/fs';
import getWebpackCommon from '../webpack/webpack.common.js';
import getWebpackDev from '../webpack/webpack.dev.js';
import WebpackDevServer from 'webpack-dev-server';
import webpack from "webpack";
import chokidar from 'chokidar';
import clearConsole from 'react-dev-utils/clearConsole';
import formatWebpackMessages from 'react-dev-utils/formatWebpackMessages';
import openBrowser from 'react-dev-utils/openBrowser';
import historyApiFallback from 'connect-history-api-fallback';
import { choosePort } from 'react-dev-utils/WebpackDevServerUtils';

const start = (config = '.luciorc.js', port = '9000', https = false, dll = false) => {
  // 初始化标识
  let init = true;
  // 读取配置文件
  const configPath = join(process.cwd(), config);

  let webpackConfig = {};
  if (exists(configPath)) {
    webpackConfig = require(configPath);
  }

  choosePort('0.0.0.0', parseInt(port)).then((port) => {
    if (port === null) {
      return;
    }

    const webpackCommon = getWebpackCommon(webpackConfig, 'development');
    const webpackDev = {...webpackCommon, ...getWebpackDev(webpackConfig, dll)};
  
    const compiler = webpack(webpackDev);
  
    compiler.plugin('invalid', () => {
      clearConsole();
      console.log('Compiling...'.bold.green);
    });
    
    compiler.plugin('done', (stats) => {
      clearConsole();
      const json = stats.toJson({}, true);
      const messages = formatWebpackMessages(json);
      const isOk = !messages.errors.length && !messages.warnings.length;
  
      if (isOk) {
        if (stats.stats) {
          console.log('Compiled successfully'.bold.green);
        } else {
          console.log(`Compiled successfully in ${(json.time / 1000).toFixed(1)}s!`.bold.green);
        }
        if (init) {
          console.log();
          console.log("                ,-.");
          console.log("      _,.      /  /");
          console.log("     ; \\____,-==-._  )    " + `The app is running at:`.green);
          console.log("     //_    `----' {+>     ");
          console.log("     `  `'--/  /-'`(       " + `  http://127.0.0.1:${port}/`.magenta);
          console.log("           /  / ");
          console.log("           `='");
          console.log();
          console.log('Note that the development build is not optimized');
          console.log(`To create a production build, use ${'lucio build'.magenta}`);
          init = false;
        }
      }
  
      // If errors exist, only show errors.
      if (messages.errors.length) {
        console.log('Failed to compile'.bold.red);
        console.log();
        messages.errors.forEach((message) => {
          console.log(message);
          console.log();
        });
  
      // Show warnings if no errors were found.
      } else if (messages.warnings.length) {
        console.log('Compiled with warnings'.bold.yellow);
        console.log();
        messages.warnings.forEach((message) => {
          console.log(message);
          console.log();
        });
        // Teach some ESLint tricks.
        console.log('You may use special comments to disable some warnings');
        console.log();
      }
    });
    // config dev server
    const devServer = new WebpackDevServer(compiler, {
      contentBase: resolve(webpackConfig.output || './dist'),
      hot: true,
      disableHostCheck: true,
      compress: true,
      quiet: true,
      stats: 'minimal',
      host: '0.0.0.0',
      open: true,
      https,
      publicPath: '/',
      proxy: webpackConfig.proxy || {},
    });
  
    // add middleware
    devServer.use(historyApiFallback({
      disableDotRule: true,
      htmlAcceptHeaders: ['text/html']
    }));
  
    devServer.use(devServer.middleware);
  
    // run dev server
    devServer.listen(port, '0.0.0.0', (err) => {
      if (err) {
        return console.log(err.bold.red);
      }
      clearConsole();
      console.log('Starting the development server...'.bold.green);
      
      openBrowser(`${https ? 'https' : 'http'}://127.0.0.1:${port}/`);
    });
  
    // watch
    const watcher = chokidar.watch(configPath, {
      ignored: /node_modules/,
      persistent: true,
    });
  
    watcher.on('change', (path) => {
      clearConsole();
      console.log(`File ${config} changed, please to restart server`.bold.yellow);
      console.log(`if you change \`${'common'.bold.yellow}\`, please run \`${'lucio dll'.bold.yellow}\``);
      console.log();
      watcher.close();
      devServer.close();
      process.exit(1);
    });
  });
}

export default start;