import Handlebars from 'handlebars';
import { join } from 'path';
import colors from 'colors';
import { read, write, exists } from './fs';
import j from 'jscodeshift';
import Collection from 'jscodeshift/dist/Collection';

// 获取模版
export const template = (src, output, data) => {
  const handlePath = join(__dirname, '../../templates/handles', src)
  if(!exists(handlePath)){
    console.log(`Error: templates file ${handlePath} not exists`.bold.red);
    process.exit(1);
  }

  const source = read(handlePath, 'utf-8');

  Handlebars.registerHelper('array', function(text, option) {
    return text.join(", ");
  });

  const template = Handlebars.compile(source);
  
  write(output, template(data), 'utf-8');
}

// 首字母大写
export const firstUpperCase  = (str) => str.toString()[0].toUpperCase() + str.toString().slice(1)

function mapImportToFile(namespace, type) {
  // 读取文件路径
  const filePath = type === 'model' ? join(process.cwd(), 'src', 'index.js') : join(process.cwd(), 'src', 'router.js');
  // import路径
  const fileImportPath = type === 'model' ? `./routes/${namespace}/model` : `./routes/${namespace}/components`;
  const fileImportName = type === 'model' ? namespace : firstUpperCase(namespace);

  if(!existsSync(filePath)){
      error(`${filePath} 不存在`);
      process.exit(1);
  }
  try {
    // 文件读取
    const fileBuffer = readFile(filePath);
    // jscodeshift初始化
    const root = j(fileBuffer);

    const imports = [];
    // 寻找全部的import节点
    root.find(j.ImportDeclaration).forEach(p => {
        const node = p.value;
        const { specifiers, source } = node;
        if (source.value.startsWith('./routes')) {
            imports.push(p);
        }
      }
    );

    const JImports = Collection.fromPaths(imports, root);
    const collection = JImports.at(0);
    // 新的import节点插入
    collection.insertAfter.call(
        collection,
        j.importDeclaration(
            [
                j.importDefaultSpecifier(
                  j.identifier(fileImportName)
                )
            ],
            j.literal(fileImportPath)
        )
    );
    return {
      filePath,
      root
    };
  } catch(e) {
    error(e);
  }
}

// 注册model
export function mapModelToIndex(namespace) {
  const {root, filePath} = mapImportToFile(namespace, 'model');
  try {
    const models = [];
    root.find(j.CallExpression).forEach(p => {
        const node = p.value;
        if (j.MemberExpression.check(node.callee)) {
            const { object, property } = node.callee;
            if(property.name === 'model') {
                models.push(p);
            }
        }
    });

    const JModels = Collection.fromPaths(models, root);
    const { object, property } = JModels.get().value.callee;
    const collection = JModels.map(node => node.parent).at(0);

    collection.insertAfter.call(
        collection, 
        j.expressionStatement(
            j.callExpression(
                j.memberExpression(object, j.identifier('model')),
                [
                    j.identifier(namespace)
                ]
            )
        )
    );
    // 文件写入
    outputFileSync(filePath, root.toSource(), 'utf-8');
  } catch(e) {
    error(e);
  }
}

// 按需加载 component and model
export function mapComponentToRouter(namespace) {
  const {root, filePath} = mapImportToFile(namespace, 'router');
  try {
    // 寻找router节点
    const findRouterNode = (root) => {
      return root.find(
        j.JSXElement, {
          openingElement: {
            name: {
              name: 'Route'
            },
          }
        }
      ).nodes()[0];
    }

    const parentRoute = findRouterNode(root);
    // 节点插入
    parentRoute.children.push(
        j.jsxElement(
            j.jsxOpeningElement(
                j.jsxIdentifier('Route'),
                [
                    j.jsxAttribute(
                        j.jsxIdentifier('path'),
                        j.literal(`/${namespace}`)
                    ),
                    j.jsxAttribute(
                        j.jsxIdentifier('component'),
                        j.jsxExpressionContainer(
                            j.jsxIdentifier(firstUpperCase(namespace))
                        )
                    ),
                ],
                true
            ),
        ),
        j.literal('\n ')
    );
    // 文件写入
    outputFileSync(filePath, root.toSource(), 'utf-8');
  } catch(e) {
    error(e);
  }
}