import { resolve } from 'path';
import webpack from 'webpack';
import autoprefixer from 'autoprefixer';

export default ({ entry = './src/index.js', output = './dist', commons = [], postcssPlugins = [], 
                  commonBabelPlugins = [], devBabelPlugins = [], prodBabelPlugins = [], resolvePlugins = {}, 
                  babelLoaderDir = [], svgSpriteDir }, env) => {
  const babelPlugins = {
    babelrc: false,
    plugins: [
      require.resolve('babel-plugin-add-module-exports'),
      require.resolve('babel-plugin-react-require'),
      require.resolve('babel-plugin-syntax-dynamic-import'),
    ].concat(commonBabelPlugins).concat(env === 'development' ? devBabelPlugins : prodBabelPlugins),
    presets:[
      require.resolve('babel-preset-es2015'),
      require.resolve('babel-preset-react'),
      require.resolve('babel-preset-stage-0'),
    ]
  };

  const postcss = [
    autoprefixer({
      browsers: [
        '>1%',
        'last 4 versions',
        'Firefox ESR',
        'not ie < 9',
      ],
    }),
    ...postcssPlugins
  ];

  const createEntry = () => {
    if (env === 'development') {
      return {
        index: [
          require.resolve('react-dev-utils/webpackHotDevClient'),
          resolve(entry),
        ]
      };
    } else {
      if (commons.length === 0) {
        return {
          index: resolve(entry)
        };
      } else {
        return {
          index: resolve(entry),
          commons: commons
        };
      }
    }
  };

  let webpackConfig = {
    entry: createEntry(),
    output: {
      filename: `[name].[${env === 'development' ? 'hash' : 'chunkhash'}].js`,
      publicPath: '/',
      path: resolve(output),
    },
    module: {
      rules: [
        {
          test: /\.(js|jsx)$/,
          include: [resolve(entry.split('/')[1]), ...babelLoaderDir],
          use: {
            loader: 'babel-loader',
            options: babelPlugins
          },
        },
        {
          test: /\.css$/,
          use: [
            'style-loader',
            'css-loader'
          ]
        },
        {
          test: /\.less$/,
          use: [
            'style-loader',
            'css-loader',
            {
              loader: 'postcss-loader',
              options: {
                plugins: postcss
              }
            },
            'less-loader'
          ]
        },
        {
          test: /\.(png|jpg|gif)$/,
          use: 'url-loader',
        },
        {
          test: /\.woff(\?v=\d+\.\d+\.\d+)?$/,
          use: {
            loader: 'url-loader',
            options: {
              limit: 8192,
              mimetype: 'application/font-woff'
            }
          }
        },
        {
          test: /\.woff2(\?v=\d+\.\d+\.\d+)?$/,
          use: {
            loader: 'url-loader',
            options: {
              limit: 8192,
              mimetype: 'application/font-woff2'
            }
          }
        },
        {
          test: /\.ttf(\?v=\d+\.\d+\.\d+)?$/,
          use: {
            loader: 'url-loader',
            options: {
              limit: 8192,
              mimetype: 'application/octet-stream'
            }
          }
        },
        { 
          test: /\.eot(\?v=\d+\.\d+\.\d+)?$/,
          use: 'file-loader'
        },
      ]
    },
    resolve: {
      extensions: ['.js', '.jsx', '.web.js'],
      ...resolvePlugins
    },
  };
  
  // add svg sprite
  if (svgSpriteDir) {
    webpackConfig.module.rules.unshift({
      test: /\.(svg)$/i, 
      use: 'svg-sprite-loader', 
      include: svgSpriteDir
    })
  };

  return webpackConfig
};